'''

Created on 2020/12

Author: Jahns_P
Version Number: 1
Date of last change: 2020/12/07
Requires: FPL1000 series SPA, FW 1.70 or newer and adequate options
- Installed RsInstrument Python module 1.7.0.37 or newer
- Installed VISA e.g. R&S Visa 5.12.x or newer

Description: Setup measurement, get trace data, slice and save it to a local CSV file


General Information:

Please always check this example script for unsuitable setting that may
destroy your DUT before connecting it to the instrument!
This example does not claim to be complete. All information has been
compiled with care. However errors can not be ruled out. 

Please find more information about RsInstrument at 
https://rsinstrument.readthedocs.io/en/latest/
'''

# --> Import necessary packets
from RsInstrument.RsInstrument import RsInstrument
from time import sleep

# Define variables
ressource='TCPIP0::10.205.0.162::hislip0::INSTR'    # VISA resource string for the device
x=0                                                 # For counters
loopResponse="1"                                    # Variable for diverse loop requests
Response="0"                                        # Variable for diverse requests
recdur = 10                                         # Time in seconds to find maxhold peaks

# Define the device handle
Instrument = RsInstrument(ressource, True, True, "SelectVisa='rs'")
'''
(ressource, True, True, "SelectVisa='rs'") has the following meaning:
(VISA-resource, id_query, reset, options)
- id_query: if True: the instrument's model name is verified against the models 
supported by the driver and eventually throws an exception.   
- reset: Resets the instrument (sends *RST) command and clears its status syb-system
- option SelectVisa:
			- 'SelectVisa = 'socket' - uses no VISA implementation for socket connections - you do not need any VISA-C installation
			- 'SelectVisa = 'rs' - forces usage of Rohde&Schwarz Visa
			- 'SelectVisa = 'ni' - forces usage of National Instruments Visa     
'''
##
## Define subroutines
##

def comprep():
    """Preparation of the communication (termination, timeout, etc...)"""
    
    print (f'VISA Manufacturer: {Instrument.visa_manufacturer}')                       # Confirm VISA package to be choosen
    Instrument.visa_timeout = 5000                                                     # Timeout in ms for VISA Read Operations
    Instrument.opc_timeout = 3000                                                      # Timeout in ms for opc-synchronised operations
    Instrument.instrument_status_checking = True                                       # Error check after each command
    Instrument.clear_status()                                                          # Clear status register
  
    
def close():
    """Close the VISA session"""
    
    Instrument.close()


def comcheck():
    """Check communication with the device by requesting it's ID"""
    
    # Just knock on the door to see if instrument is present
    idnResponse = Instrument.query_str('*IDN?')
    sleep (1)
    print ('Hello, I am ' + idnResponse)
    
   
def measprep():
    """Prepare instrument for desired measurements
    In this case
    - Set Center Frequency to 1540 MHz
    - Set Span to 100 MHz
    - Set Trace to Max Hold (and Positive Peak automatically)
    """

    Instrument.write_str_with_opc('FREQuency:CENTer 2450e6')                    # Center Frequency to 2450 MHz
    Instrument.write_str_with_opc('FREQuency:SPAN 100e6')                       # SPAN is 100 MHz now
    Instrument.write_str_with_opc('DISPlay:TRACe1:MODE MAXHold')                # Trace to Max Hold

def traceget():
    """Initialize continuous measurement, stop it after 10 seconds and query trace data"""

    Instrument.write_str_with_opc('INITiate:CONTinuous ON')                     # Continuous measurement on trace 1 ON
    sleep (int(recdur))                                                         # Wait for preset record time
    Instrument.write_str_with_opc('INITiate:CONTinuous OFF')                    # Continuous measurement on trace 1 OFF
    TraceData=Instrument.query_str('Trace:DATA? TRACe1')                        # Read y data of trace 1
    TracePoints=Instrument.query_str('TRACe:DATA:X? Trace1')                    # Read the corresponding x data
    NoSweepPoints = Instrument.query_str('SWEep:POINts?')                       # Request for the number of sweep points

    CSVTraceData=TraceData.split(",")                                           # Slice the amplitude list into a tuple
    CSVTracePoints=TracePoints.split(",")                                       # Slice the frequency list into a tuple

    # now file writing begins
    file = open (r'C:\test\TraceFile.CSV', 'w')                                 # Open File for writing
    file.write("Frequency in Hz;Power in dBm\n")                                # Write the headline
    x=0                                                                         # Set counter to 0 as touple does not begin with 1
    while x < int(NoSweepPoints):                                               # Perform loop until all sweep points are covered
        file.write(CSVTracePoints [x])                                          # Write amplitude measurement data
        file.write(";")                                                         # Add semicolon as CSV separator
        file.write(CSVTraceData [x])                                            # Write frequency data
        file.write("\n")                                                        # Add a new line control character
        x=x+1                                                                   # Increment counter
    file.close()                                                                # CLose the file
    
##
##-------------------------
## Main Program begins here 
##-------------------------
##
    
comprep()
comcheck()
measprep()
traceget()
close()


print ("Programm succesfully ended.")